/* Standard headers */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <stdarg.h>
#include <sys/types.h>
#include <sys/disk.h>
#include <sys/vc.h>
#include <sys/osinfo.h>
#include <process.h>
#include <sys/qnx_glob.h>
#include <sys/psinfo.h>
#include <sys/kernel.h>


/* Toolkit headers */
#include <Ph.h>
#include <Pt.h>
#include <Ap.h>

/* Local headers */
#include "life.h"
#include "abimport.h"
#include "proto.h"

/*--- -----------------------------------------------------------------	---*/
/*--- Function: message											 		---*/
/*--- 															 		---*/
/*--- Usage:	Print a message inside a dialog window. Just ok. 		---*/
/*--- -----------------------------------------------------------------	---*/
int message( const char *title, const char *fmt, ... )
	{
	va_list ap;
	char    buff[ 384 ];

	va_start( ap, fmt );
	vsprintf( buff, fmt, ap );
	va_end( ap );

	PtMessageBox( ABW_zoom, title, buff, "helv14.phf", "  OK  " );

	return -1;

	}	// message


/*--- -----------------------------------------------------------------	---*/
/*--- Function: app_calloc										 		---*/
/*--- 															 		---*/
/*--- Usage:	Same as calloc, but if allocation fails, exit the app.	---*/
/*--- -----------------------------------------------------------------	---*/
void * app_calloc( const size_t nb, const size_t sz )
	{
	void *p;
	
	if ( (p = calloc( nb, sz )) != NULL )
		return p;
		
	fprintf( stderr, "Not enought memory (allocation of %d bytes has failed)\n",
		nb * sz );

	message( "Error", "Not enought memory\n(allocation of %d bytes has failed)", 
		nb * sz );

	exit( -1 );
	return NULL;

	}	// app_calloc


/*--- -----------------------------------------------------------------	---*/
/*--- Function: app_realloc										 		---*/
/*--- 															 		---*/
/*--- Usage:	Same as calloc, but if allocation fails, exit the app.	---*/
/*--- -----------------------------------------------------------------	---*/
void * app_realloc( void **ptr, const size_t sz )
	{
	void *p;
	
	if ( (p = realloc( *ptr, sz )) != NULL )
		return *ptr = p;
		
	fprintf( stderr, "Not enought memory (allocation of %d bytes has failed)\n",
		sz );

	message( "Error", "Not enought memory\n(allocation of %d bytes has failed)", 
		sz );

	exit( -1 );
	return NULL;

	}	// app_realloc


/*--- -----------------------------------------------------------------	---*/
/*--- Function: app_malloc										 		---*/
/*--- 															 		---*/
/*--- Usage:	Same as malloc, but if allocation fails, exit the app.	---*/
/*--- -----------------------------------------------------------------	---*/
void * app_malloc( const size_t sz )
	{
	void *p;
	
	if ( (p = malloc( sz )) != NULL )
		return p;
		
	fprintf( stderr, "Not enought memory (allocation of %d bytes has failed)\n", 
		sz );

	message( "Error", "Not enought memory\n(allocation of %d bytes has failed)", 
		sz );

	exit( -1 );
	return NULL;

	}	// app_malloc


/*--- -----------------------------------------------------------------	---*/
/*--- Function: setBtn											 		---*/
/*--- 															 		---*/
/*--- Usage:	Put the given button widget into the set/non set state.	---*/
/*--- -----------------------------------------------------------------	---*/
void setBtn( const PtWidget_t *wgt, const BOOL state )
	{
	PtArg_t	args[ 1 ];

	PtSetArg( &args[ 0 ], Pt_ARG_FLAGS, (state) ? Pt_TRUE : Pt_FALSE, Pt_SET );
	PtSetResources( wgt, 1, args );
	}	// setBtn


/*--- -----------------------------------------------------------------	---*/
/*--- Function: isBtnSet										 		---*/
/*--- 															 		---*/
/*--- Usage:	Is a given button widget into the set state ?			---*/
/*--- -----------------------------------------------------------------	---*/
BOOL isBtnSet( const PtWidget_t *wgt )
	{
	PtArg_t			args[ 1 ];
	unsigned long	*flags;

	PtSetArg( &args[ 0 ], Pt_ARG_FLAGS, &flags, 0 );
	PtGetResources( wgt, 1, args );
	return (BOOL) ((*flags & Pt_SET) == Pt_SET);
	}	// isBtnSet


/*--- -----------------------------------------------------------------	---*/
/*--- Function: getWhichBtnSet									 		---*/
/*--- 															 		---*/
/*--- Usage:	In a given list of buttons,which one is the set state ?	---*/
/*--- 			The return value is in the range: 1, 2, ...				---*/
/*--- 			The list of buttons must be termined by a NULL.			---*/
/*--- -----------------------------------------------------------------	---*/
int getWhichBtnSet( const PtWidget_t *wgt0, ... )
	{
	PtWidget_t	*wgt;
	va_list		arg;
	int			no;

	va_start( arg, wgt0 );
	for ( wgt = wgt0, no = 1; wgt; no++ )
		{
		if ( isBtnSet( wgt ) )
			{
			va_end( arg );
			return no;
			}
		wgt = va_arg( arg, PtWidget_t * );
		}
	va_end( arg );
	return -1;

	}	// getWhichBtnSet


/*--- -----------------------------------------------------------------	---*/
/*--- Function: setWhichBtnSet									 		---*/
/*--- 															 		---*/
/*--- Usage:	In a given list of buttons, set a given one to ON.		---*/
/*--- 			The list of buttons must be termined by a NULL.			---*/
/*--- -----------------------------------------------------------------	---*/
void setWhichBtnSet( const int setNo, ... )
	{
	PtWidget_t	*wgt;
	va_list		arg;
	int			no;

	va_start( arg, setNo);
	wgt = va_arg( arg, PtWidget_t * );
	for ( no = 1; wgt; no++ )
		{
		setBtn( wgt, (no == setNo) );
		wgt = va_arg( arg, PtWidget_t * );
		}
	va_end( arg );

	}	// setWhichBtnSet


/*--- -----------------------------------------------------------------	---*/
/*--- Function: setOnOffBtn										 		---*/
/*--- 															 		---*/
/*--- Usage:	Set the given button into the on/off state.				---*/
/*--- -----------------------------------------------------------------	---*/
void setOnOffBtn( const PtWidget_t *wgt, const BOOL state )
	{
	PtArg_t	args[ 1 ];

	PtSetArg( &args[ 0 ], Pt_ARG_ONOFF_STATE, (state) ? 1 : 0, 0 );
	PtSetResources( wgt, 1, args );
	}	// setOnOffBtn


/*--- -----------------------------------------------------------------	---*/
/*--- Function: isOnOffBtnSet									 		---*/
/*--- 															 		---*/
/*--- Usage:	Is the given button into the on (vs off) state ?		---*/
/*--- -----------------------------------------------------------------	---*/
BOOL isOnOffBtnSet( const PtWidget_t *wgt )
	{
	PtArg_t		args[ 1 ];
	ushort_t	*state;

	PtSetArg( &args[ 0 ], Pt_ARG_ONOFF_STATE, &state, 0 );
	PtGetResources( wgt, 1, args );
	return (*state) ? TRUE : FALSE;
	}	// isOnOffBtnSet


/*--- -----------------------------------------------------------------	---*/
/*--- Function: ghostBtn										 		---*/
/*--- 															 		---*/
/*--- Usage:	Ghost (both aspect and behaviour) a button.				---*/
/*--- -----------------------------------------------------------------	---*/
int ghostBtn( const PtWidget_t *wgt, const BOOL state )
	{
	PtArg_t	args[ 3 ];

	PtSetArg( &args[ 0 ], Pt_ARG_FLAGS, (state) ? Pt_FALSE : Pt_TRUE, Pt_SELECTABLE );
	PtSetArg( &args[ 1 ], Pt_ARG_FLAGS, (state) ? Pt_TRUE : Pt_FALSE, Pt_GHOST );
	PtSetArg( &args[ 2 ], Pt_ARG_FLAGS, (state) ? Pt_TRUE : Pt_FALSE, Pt_BLOCKED );
	PtSetResources( wgt, 3, args );
	return 0;

	}	// ghostBtn


/*--- -----------------------------------------------------------------	---*/
/*--- Function: setString										 		---*/
/*--- 															 		---*/
/*--- Usage:	Change the text_string of widgets.						---*/
/*--- -----------------------------------------------------------------	---*/
void setString( const PtWidget_t *wgt, const char *fmt, ... )
	{
	PtArg_t		args[ 1 ];
	va_list		ap;
	char		buff[ 200 ];

	va_start( ap, fmt );
	vsprintf( buff, fmt, ap );
	va_end( ap );

	PtSetArg( &args[ 0 ], Pt_ARG_TEXT_STRING, buff, 0 );
	PtSetResources( wgt, 1, args );
	}	// setString


/*--- -----------------------------------------------------------------	---*/
/*--- Function: get_string										 		---*/
/*--- 															 		---*/
/*--- Usage:	Read the string of a given widget.						---*/
/*--- -----------------------------------------------------------------	---*/
void getString( const PtWidget_t *wgt, char *string )
	{
	PtArg_t	args[ 1 ];
	char	*value;

	PtSetArg( &args[ 0 ], Pt_ARG_TEXT_STRING, &value, 0 );
	PtGetResources( wgt, 1, args );
	strcpy( string, value );
	}	// getString


// ----------------------------------------------------------------------------
// Change the title of a window
// ----------------------------------------------------------------------------
void setWindowTitle( const PtWidget_t *wgt, const char *fmt, ... )
	{
	PtArg_t		args[ 1 ];
	va_list		ap;
	char		buff[ 200 ];

	va_start( ap, fmt );
	vsprintf( buff, fmt, ap );
	va_end( ap );

	PtSetArg( &args[ 0 ], Pt_ARG_WINDOW_TITLE, buff, 0 );
	PtSetResources( wgt, 1, args );

	}	// setWindowTitle


/*--- -----------------------------------------------------------------	---*/
/*--- Function: set_color										 		---*/
/*--- 															 		---*/
/*--- Usage:	Change the color of the given widget.					---*/
/*--- -----------------------------------------------------------------	---*/
void setColor( const PtWidget_t *wgt, const PgColor_t color )
	{
	PtArg_t	args[ 1 ];

	PtSetArg( &args[ 0 ], Pt_ARG_COLOR, color, 0 );
	PtSetResources( wgt, 1, args );

	}	// setColor


/*--- -----------------------------------------------------------------	---*/
/*--- Function: blockWindow												---*/
/*--- 															 		---*/
/*--- Usage:	Set the given window into the blocked state.			---*/
/*--- -----------------------------------------------------------------	---*/
int blockWindow( const PtWidget_t *wgt, const BOOL state )
	{
	PtArg_t	args[ 1 ];

	PtSetArg( &args[ 0 ], Pt_ARG_FLAGS, (state) ? Pt_TRUE : Pt_FALSE, Pt_BLOCKED );
	PtSetResources( wgt, 1, args );

	return Pt_CONTINUE;
	}	// blockWindow


/*--- -----------------------------------------------------------------	---*/
/*--- Function: setGauge_MinMax											---*/
/*--- 															 		---*/
/*--- Usage:	Ste the min/max/curr of the given gauge widget.			---*/
/*--- -----------------------------------------------------------------	---*/
void setGauge_MinMax( const PtWidget_t *wgt, 
					  const int vmin, const int vmax, const int vcurr )
	{
	PtArg_t	args[ 3 ];

	PtSetArg( &args[ 0 ], Pt_ARG_GAUGE_MINIMUM, vmin, 0 );
	PtSetArg( &args[ 1 ], Pt_ARG_GAUGE_MAXIMUM, vmax, 0 );
	PtSetArg( &args[ 2 ], Pt_ARG_GAUGE_VALUE,   vcurr, 0 );
	PtSetResources( wgt, 3, args );
	PtFlush();

	}	// setGauge_MinMax


/*--- -----------------------------------------------------------------	---*/
/*--- Function: getGauge_value											---*/
/*--- 															 		---*/
/*--- Usage:	Get the current value of the given gauge widget.		---*/
/*--- -----------------------------------------------------------------	---*/
int getGauge_value( const PtWidget_t *wgt )
	{
	PtArg_t	args[ 1 ];
	int		*value;

	PtSetArg( &args[ 0 ], Pt_ARG_GAUGE_VALUE, &value, 0 );
	PtGetResources( wgt, 1, args );
	return *value;

	}	// getGauge_value


/*--- -----------------------------------------------------------------	---*/
/*--- Function: setGauge_value											---*/
/*--- 															 		---*/
/*--- Usage:	Set the current value of the given gauge widget.		---*/
/*--- -----------------------------------------------------------------	---*/
void setGauge_value( const PtWidget_t *wgt, const int value )
	{
	PtArg_t	args[ 1 ];

	PtSetArg( &args[ 0 ], Pt_ARG_GAUGE_VALUE, value, 0 );
	PtSetResources( wgt, 1, args );
	PtFlush();

	}	// setGauge_value


/*--- -----------------------------------------------------------------	---*/
/*--- Function: getGauge_value											---*/
/*--- 															 		---*/
/*--- Usage:	Get the current value of the given gauge widget.		---*/
/*--- -----------------------------------------------------------------	---*/
void setCursorType( const PtWidget_t *wgt, int type )
	{
	PtArg_t	args[ 1 ];

	PtSetArg( &args[ 0 ], Pt_ARG_CURSOR_TYPE, type, 0 );
	PtSetResources( wgt, 1, args );

	}	// setCursorType


/*--- -----------------------------------------------------------------	---*/
/*--- Function: busyWindow												---*/
/*--- 															 		---*/
/*--- Usage:	Put the given window into the 'busy' state.				---*/
/*--- -----------------------------------------------------------------	---*/
int busyWindow( const PtWidget_t *wgt, const BOOL state )
	{
	blockWindow( wgt, state );
	setCursorType( wgt, 
		(state) ? Ph_CURSOR_CLOCK : Ph_CURSOR_POINTER );
	PtFlush();
	return Pt_CONTINUE;
	}	// busyWindow


/*--- -----------------------------------------------------------------	---*/
/*--- Function: addListItem												---*/
/*--- 															 		---*/
/*--- Usage:	Add a new line at the end of a list widget				---*/
/*--- -----------------------------------------------------------------	---*/
int addListItem( const PtWidget_t *wgt, const char *fmt, ... )
	{
	va_list		ap;
	char		buff[ 200 ];
	char		*items[ 1 ];

	/*--- Develop arguments ---*/
	va_start( ap, fmt );
	vsprintf( buff, fmt, ap );
	va_end( ap );

	/*--- Add this new line ---*/
	items[ 0 ] = buff;
	return PtListAddItems( wgt, items, 1, 0 );

	}	// addListItem


/*--- -----------------------------------------------------------------	---*/
/*--- Function: getDim													---*/
/*--- 															 		---*/
/*--- Usage:	Get the dimension of a widget							---*/
/*--- -----------------------------------------------------------------	---*/
void getDim( const PtWidget_t *wgt, USHORT *w, USHORT *h )
	{
	PtArg_t		args[ 1 ];
	PhDim_t		*dim;

	PtSetArg( &args[ 0 ], Pt_ARG_DIM, &dim, 0 );
	PtGetResources( wgt, 1, args );
	if ( w )
		*w = dim->w;
	if ( h )
		*h = dim->h;

	}	// getDim


/*--- -----------------------------------------------------------------	---*/
/*--- Function: setDim													---*/
/*--- 															 		---*/
/*--- Usage:	Change the dimension of the given widget.				---*/
/*--- -----------------------------------------------------------------	---*/
void setDim( const PtWidget_t *wgt, const int w, const int h )
	{
	PtArg_t		args[ 1 ];
	PhDim_t		size;

	size.w = w;
	size.h = h;
	PtSetArg( &args[ 0 ], Pt_ARG_DIM, &size, 0 );
	PtSetResources( wgt, 1, args );

	}	// setDim


/*--- -----------------------------------------------------------------	---*/
/*--- Function: getPosWidget											---*/
/*--- 															 		---*/
/*--- Usage:	Get the position of a widget							---*/
/*--- -----------------------------------------------------------------	---*/
void getPosWidget( const PtWidget_t *wgt, int *x, int *y )
	{
	PtArg_t		args[ 1 ];
	PhArea_t	*area;

	PtSetArg( &args[ 0 ], Pt_ARG_AREA, &area, 0 );
	PtGetResources( wgt, 1, args );
	if ( x )
		*x = area->pos.x;
	if ( y )
		*y = area->pos.y;

	}	// getPosWidget


/*--- -----------------------------------------------------------------	---*/
/*--- Function: setPosWidget											---*/
/*--- 															 		---*/
/*--- Usage:	Set the position of a widget							---*/
/*--- -----------------------------------------------------------------	---*/
void setPosWidget( const PtWidget_t *wgt, const int x, const int y )
	{
	PtArg_t		args[ 1 ];
	PhPoint_t	point;

	point.x = x;
	point.y = y;
	PtSetArg( &args[ 0 ], Pt_ARG_POS, &point, 0 );
	PtSetResources( wgt, 1, args );

	}	// setPosWidget


/*--- -----------------------------------------------------------------	---*/
/*--- Function: moveByWidget											---*/
/*--- 															 		---*/
/*--- Usage:	Set the position of a widget							---*/
/*--- -----------------------------------------------------------------	---*/
void moveByWidget( PtWidget_t *wgt, const int dx, const int dy )
{
	PhPoint_t	*old_pos, new_pos;
	PtArg_t		args[ 1 ];

	PtSetArg( &args[ 0 ], Pt_ARG_POS, &old_pos, 0 );
	PtGetResources( wgt, 1, args );
	memmove( &new_pos, old_pos, sizeof( PhPoint_t ) );
	new_pos.x += dx;
	new_pos.y += dy;
	PtSetArg( &args[ 0 ], Pt_ARG_POS, &new_pos, 0 );
	PtSetResources( wgt, 1, args );
	}	// moveByWidget


/*--- -----------------------------------------------------------------	---*/
/*--- Function: getInt													---*/
/*--- 															 		---*/
/*--- Usage:	Read the current numeric value of the given widget.		---*/
/*--- -----------------------------------------------------------------	---*/
int getInt( const PtWidget_t *wgt )
	{
	PtArg_t	args[ 1 ];
	int		*currVal;

	PtSetArg( &args[ 0 ], Pt_ARG_NUMERIC_VALUE, &currVal, 0 );
	PtGetResources( wgt, 1, args );
	return *currVal;

	}	// getInt


/*--- -----------------------------------------------------------------	---*/
/*--- Function: setInt													---*/
/*--- 															 		---*/
/*--- Usage:	Set the current numeric value of the given widget.		---*/
/*--- -----------------------------------------------------------------	---*/
void setInt( const PtWidget_t *wgt, const int val )
	{
	PtArg_t	args[ 1 ];

	PtSetArg( &args[ 0 ], Pt_ARG_NUMERIC_VALUE, val, 0 );
	PtSetResources( wgt, 1, args );

	}	// setInt


/*--- -----------------------------------------------------------------	---*/
/*--- Function: addItem_list											---*/
/*--- 															 		---*/
/*--- Usage:	Add a new line at the end of a list widget				---*/
/*--- -----------------------------------------------------------------	---*/
int addItem_list( const PtWidget_t *wgt, 
				  const BOOL is_horo, const int max_nb_kept, const int nb_seen,
				  const char *fmt, ... )
	{
	va_list 	ap;
	char    	buff[ 384 ], *items[ 1 ];
	PtArg_t		args[ 1 ];
	int			top;
	time_t		now;
	struct tm	*tm;
	short		*nbElemInList;

	/*--- Develop arguments ---*/
	va_start( ap, fmt );
	if ( is_horo )
		{
		time( &now );
		tm = localtime( &now );
		sprintf( buff, "%02d:%02d:%02d ",
			tm->tm_hour, tm->tm_min, tm->tm_sec );
		vsprintf( &buff[ 9 ], fmt, ap );
		}
	else
		vsprintf( buff, fmt, ap );
	va_end( ap );

	/*--- Get the number of current elements into the list ---*/
	PtSetArg( &args[ 0 ], Pt_ARG_LIST_ITEM_COUNT, &nbElemInList, 0 );
	PtGetResources( wgt, 1, args );

	/*--- Delete the first element ? ---*/
	if ( *nbElemInList >= max_nb_kept )
		PtListDeleteItemPos( wgt, 1, 1 );

	/*--- Add the new element ---*/
	items[ 0 ] = buff;
	PtListAddItems( wgt, items, 1, 0 );

	/*--- Scroll the list to see the last element added ---*/
	if ( *nbElemInList > nb_seen )
		{
		top = *nbElemInList - (nb_seen - 1);
		PtSetArg( &args[ 0 ], Pt_ARG_TOP_ITEM_POS, &top, 0 );
		PtSetResources( wgt, 1, args );
		}

	PtFlush();
	return 0;
	}	// addItem_list


void window_focus( PtWidget_t *wgt )
	{
	PhWindowEvent_t		we;

	/*--- Send appropriate message to the window manager ---*/
	memset( &we, 0, sizeof( PhWindowEvent_t ) );
	we.rid         = PtWidgetRid( wgt );
	we.event_f     = Ph_WM_FOCUS;
	we.event_state = Ph_WM_EVSTATE_FOCUS;
	PtForwardWindowEvent( &we );
	
	}	// window_focus


void iconify_window( PtWidget_t *wgt, const BOOL icon )
	{
	PhWindowEvent_t		we;

	/*--- Nothing to do is window is not 'mounted' ---*/
	if ( wgt == NULL )
		return;

	/*--- Send appropriate message to the window manager ---*/
	memset( &we, 0, sizeof( PhWindowEvent_t ) );
	we.rid         = PtWidgetRid( wgt );
	if ( icon)
		{
	
		/*--- Set the zoom window into the normal ---*/
		we.event_f     = Ph_WM_ICON;
		we.event_state = Ph_WM_EVSTATE_ICONIFY;

		}
	else
		{
	
		/*--- Set the zoom window into the normal ---*/
		we.event_f     = Ph_WM_RESTORE;
		we.event_state = Ph_WM_EVSTATE_DEICONIFY;

		}
	PtForwardWindowEvent( &we );

	if ( icon )
		window_focus( wgt );
	
	}	// iconify_window


/*--- -----------------------------------------------------------------	---*/
/*--- Function: get_font										 		---*/
/*--- 															 		---*/
/*--- Usage:	Read the font of a given widget.						---*/
/*--- -----------------------------------------------------------------	---*/
void getFont( const PtWidget_t *wgt, char *string )
	{
	PtArg_t	args[ 1 ];
	char	*value;

	PtSetArg( &args[ 0 ], Pt_ARG_TEXT_FONT, &value, 0 );
	PtGetResources( wgt, 1, args );
	strcpy( string, value );
	}	// getFont


/*--- -----------------------------------------------------------------	---*/
/*--- Function: get_font										 		---*/
/*--- 															 		---*/
/*--- Usage:	Read the font of a given widget.						---*/
/*--- -----------------------------------------------------------------	---*/
void getTextExtent( const PtWidget_t *wgt, PhRect_t *extent )
	{
	char		font[ 80 ];
	PtArg_t		args[ 1 ];
	char		*text;

	/*--- Get the font used by the (label) widget ---*/
	getFont( wgt, font );

	/*--- Get the text extent ---*/	
	PtSetArg( &args[ 0 ], Pt_ARG_TEXT_STRING, &text, 0 );
	PtGetResources( wgt, 1, args );
	PfExtentText( extent, NULL, font, text, 0 );
	
	}	// getTextExtent


void get_rel_possiz_window( PtWidget_t *wgt, int *x, int *y, int *w, int *h )
	{
	PhArea_t	*area;
	PtArg_t		args[ 1 ];
	PhRect_t	r;

	/*--- Ask for the size and the absolute position ---*/
	PtSetArg( &args[ 0 ], Pt_ARG_AREA, &area, 0 );
	PtGetResources( wgt, 1, args );
	*w = area->size.w;
	*h = area->size.h;

	/*--- Translate into relative position to 1st console ---*/
	PhWindowQueryVisible( Ph_QUERY_GRAPHICS, 0, 1, &r );
	if (r.ul.x)	*x = area->pos.x % r.ul.x;
	else		*x = area->pos.x;
	if (r.ul.y)	*y = area->pos.y % r.ul.y;
	else		*y = area->pos.y;

	}	// get_rel_possiz_window


void set_possiz_window( PtWidget_t *wgt, const int x, const int y, const int w, const int h )
	{
	PhArea_t	area;
	PtArg_t		args[ 1 ];
	PhRect_t	r;

	/*--- Translate into relative position to 1st console ---*/
	PhWindowQueryVisible( Ph_QUERY_IG_POINTER, 0, 1, &r );
	area.pos.x = x + r.ul.x;
	area.pos.y = y + r.ul.y;

	/*--- Ask for the size and the absolute position ---*/
	area.size.w = w;
	area.size.h = h;
	PtSetArg( &args[ 0 ], Pt_ARG_AREA, &area, 0 );
	PtSetResources( wgt, 1, args );

	}	// set_possiz_window
